@@ -38,7 +38,7 @@ setClass(
       } else {
         command = paste(command, ",")
       }
-      if (!is.null(elem) && !is.na(elem)) {
+      if (!is.null(elem)) {
         command = paste(command, "c(")
         first = T
         for (val in elem) {
@@ -138,10 +138,53 @@ setClass(
   return(data.frame(columns = columns, fun = fun, level = lvl))
 }
 
-#' Shows a \code{Cube} object
+.entropy = function(x) {
+  freq = table(x) / length(x)
+  h = -sum(freq * log2(freq))
+  return(h)
+}
+
+.informationGain = function(feature, data, target) {
+  data = data[!is.na(data[,feature]),]
+  classH = .entropy(data[,target])
+  if (is.factor(data[,feature])) {
+    featureData = data %>% group_by_at(feature) %>% summarise(e = .entropy(get(target)), n = length(get(target)))
+  } else {
+    bins = length(unique(data[,feature]))
+    data$cat = cut(data[,feature], breaks=bins, labels=c(1:bins))
+    featureData = data %>% group_by(cat) %>% summarise(e = .entropy(get(target)), n = length(get(target)))
+  }
+  featureData$p = featureData$n/nrow(data)
+  gain = classH - sum(featureData$p * featureData$e)
+  return(gain)
+}
+
+.dimensionImportance = function(x, imp) {
+  values = imp[which(rownames(imp) == x),]
+  dim(values) = c(length(x), 1)
+  rownames(values) = rownames(imp)[which(rownames(imp) == x)]
+  colnames(values) = "importance"
+  dimImportance = list(importance = sum(values), attributes = values)
+  return(dimImportance)
+}
+
+.convert = function(x) {
+  if (is.factor(x)) {
+    y = tryCatch({
+      x = as.numeric(as.character(x))
+    }, warning = function(w) {
+      x
+    })
+  } else {
+    y = x
+  }
+  return(y)
+}
+
+#' Shows a Cube object
 #'
 #' Shows the actual view of a \code{Cube} object. All added selections and aggregations will be
-#' regarded. Note that selection criteria will be applied before the
+#' regarded. Note that selection criteria will be applied before
 #' aggregating the data.
 #'
 #' @param object The \code{Cube} object
@@ -183,6 +226,51 @@ setMethod("show", "Cube", function(object) {
 })
 
 
+#' Visualizes a Cube object as parallel coordinate plot
+#'
+#' Generates a parallel coordinate plot for a given \code{Cube} object. All added selections and aggregations will be
+#' regarded. 
+#'
+#' @param x The \code{Cube} object that should be plotted.
+#' @param color The color of the lines in the parallel coordinate plot. If this parameter is NA or NULL, a colorscale rather than a unique color will be used.
+#' @param colorscale The colorscale for the lines in the parallel coordinate plot. Default is RdBu. All plotly colorscales (e.g., Blackbody, Earth, Jet) are possible.
+#' @param ... Further plot_ly parameters.
+#' @author Michael Scholz \email{michael.scholz@@uni-passau.de}
+#' @seealso \code{\link[=Cube-class]{Cube}}
+#' @keywords methods
+#' @docType methods
+#' @examples
+#'
+#' data("sales")
+#' cube = generateCube(sales, columns = list(time = c("month", "year"),
+#'       location = c("state"), product = "product"), valueColumn = "amount")
+#' plot(cube)
+#'
+#' @export
+setMethod("plot", "Cube", function(x, color = NA, colorscale = "RdBu", ...) {
+  df = as.data.frame(x)
+  nas = which(is.na(df[,ncol(df)]))
+  if (length(nas) > 0) {
+    df = df[-nas,]
+  }
+  dimensions = list()
+  for (i in 1:ncol(df)) {
+    value = .convert(df[,i])
+    if (is.factor(value)) {
+      dimensions[[i]] = list(range = c(min(as.numeric(value), na.rm = T), max(as.numeric(value), na.rm = T)), label = names(df)[i], values = as.numeric(value), tickvals = unique(as.numeric(value)), ticktext = levels(value))
+    } else {
+      dimensions[[i]] = list(range = c(min(value, na.rm = T), max(value, na.rm = T)), label = names(df)[i], values = value)
+    }
+  }
+  if (is.na(color) || is.null(color)) {
+    p = plot_ly(type = "parcoords", line = list(color = ~value, colorscale = colorscale), dimensions = dimensions, ...)
+  } else {
+    p = plot_ly(type = "parcoords", line = list(color = color, colorscale = colorscale), dimensions = dimensions, ...)
+  }
+  return(p)
+})
+
+
 #' Generates a hypercube from a given dataframe
 #'
 #' This function generates a hypercube from a given dataframe. The dimensions of the
@@ -195,6 +283,8 @@ setMethod("show", "Cube", function(object) {
 #' @return Returns a \code{Cube} object.
 #' @author Michael Scholz \email{michael.scholz@@uni-passau.de}
 #' @seealso \code{\link[=Cube-class]{Cube}}
+#' @keywords methods
+#' @docType methods
 #' @examples
 #'
 #' data("sales")
@@ -227,14 +317,17 @@ generateCube = function(data, columns, valueColumn) {
 #'
 #' This function adds further selection criteria to a hypercube.
 #' The cube itself will not be changed. The selection criteria only affect the data that
-#' will be shown when printing the cube. Note that selection criteria will be applied before the
+#' will be shown when printing the cube. Note that selection criteria will be applied before
 #' aggregating the data.
 #'
-#' @param cube Hypercube for which the selection criteria will be defined.
+#' @param x Hypercube for which the selection criteria will be defined.
 #' @param criteria A list of selection criteria.
 #' @return Returns a \code{Cube} object with the added selection criteria.
 #' @author Michael Scholz \email{michael.scholz@@uni-passau.de}
 #' @seealso \code{\link[=Cube-class]{Cube}} \code{\link{remove.selection}} \code{\link{add.aggregation}}
+#' @aliases add.selection,Cube-method
+#' @keywords methods
+#' @docType methods
 #' @examples
 #'
 #' data("sales")
@@ -247,32 +340,38 @@ generateCube = function(data, columns, valueColumn) {
 #' cube
 #'
 #' @export add.selection
-add.selection = function(cube, criteria) {
-  if (!is.object(cube)) {
-    stop("Parameter cube must be of type Cube.")
+setGeneric("add.selection", function(x, criteria)
+  standardGeneric("add.selection"))
+setMethod("add.selection", "Cube", function(x, criteria) {
+  if (!is.object(x)) {
+    stop("Parameter x must be of type Cube.")
   }
-  if (class(cube)[[1]] != "Cube") {
-    stop("Parameter cube must be of type Cube.")
+  if (class(x)[[1]] != "Cube") {
+    stop("Parameter x must be of type Cube.")
   }
   if (!is.list(criteria)) {
     stop("Parameter criteria must be a list.")
   } else {
-    dims = as.character(unlist(cube@structure))
+    dims = as.character(unlist(x@structure))
     for (criterion in names(criteria)) {
       if (criterion %in% dims) {
-        if (sum(criteria[[criterion]] %in% dimnames(cube@data)[[criterion]]) == length(criteria[[criterion]])) {
-          cube@view[[criterion]]@values = unique(c(cube@view[[criterion]]@values ,criteria[[criterion]]))
-          cube@view[[criterion]]@values = cube@view[[criterion]]@values[which(cube@view[[criterion]]@values != "")]
+        if (!is.null(criteria[[criterion]])) {
+          if (sum(criteria[[criterion]] %in% dimnames(x@data)[[criterion]]) == length(criteria[[criterion]])) {
+            x@view[[criterion]]@values = unique(c(x@view[[criterion]]@values ,criteria[[criterion]]))
+            x@view[[criterion]]@values = x@view[[criterion]]@values[which(x@view[[criterion]]@values != "")]
+          } else {
+            stop(paste("There is no level", criteria[[criterion]], "in dimension", criterion, "."))
+          }
         } else {
-          stop(paste("There is no level", criteria[[criterion]], "in dimension", criterion, "."))
+          stop(paste("Criterion", criterion, "is null."))
         }
       } else {
         stop(paste("There is no dimension", criterion, "in your cube."))
       }
     }
   }
-  return(cube)
-}
+  return(x)
+})
 
 
 #' Removes selection criteria from a hypercube
@@ -281,11 +380,14 @@ add.selection = function(cube, criteria) {
 #' The cube itself will not be changed. The selection criteria only affect the data that
 #' will be shown when printing the cube.
 #'
-#' @param cube Hypercube for which the selection criteria will be defined.
+#' @param x Hypercube for which the selection criteria will be defined.
 #' @param dimensions A vector of dimension names for which all selection criteria will be removed.
 #' @return Returns a \code{Cube} object with removed selection criteria.
 #' @author Michael Scholz \email{michael.scholz@@uni-passau.de}
 #' @seealso \code{\link[=Cube-class]{Cube}} \code{\link{add.selection}} \code{\link{remove.aggregation}}
+#' @aliases remove.selection,Cube-method
+#' @keywords methods
+#' @docType methods
 #' @examples
 #'
 #' data("sales")
@@ -298,41 +400,46 @@ add.selection = function(cube, criteria) {
 #' cube
 #'
 #' @export remove.selection
-remove.selection = function(cube, dimensions) {
-  if (!is.object(cube)) {
-    stop("Parameter cube must be of type Cube.")
+setGeneric("remove.selection", function(x, dimensions)
+  standardGeneric("remove.selection"))
+setMethod("remove.selection", "Cube", function(x, dimensions) {
+  if (!is.object(x)) {
+    stop("Parameter x must be of type Cube.")
   }
-  if (class(cube)[[1]] != "Cube") {
-    stop("Parameter cube must be of type Cube.")
+  if (class(x)[[1]] != "Cube") {
+    stop("Parameter x must be of type Cube.")
   }
   if (!is.vector(dimensions)) {
     stop("Parameter dimensions must be a vector.")
   }
-  dims = unlist(cube@structure)
+  dims = unlist(x@structure)
   for (dimension in dimensions) {
     if (dimension %in% dims) {
-      cube@view[[dimension]]@values = ""
+      x@view[[dimension]]@values = ""
     } else {
       stop(paste("There is no dimension", dimension, "in your cube."))
     }
   }
-  return(cube)
-}
+  return(x)
+})
 
 
 #' Adds an aggregation to a hypercube
 #'
 #' This function adds a further aggregation to a hypercube.
 #' The cube itself will not be changed. The aggregation only affect the data that
-#' will be shown when printing the cube. Note that selection criteria will be applied before the
+#' will be shown when printing the cube. Note that selection criteria will be applied before
 #' aggregating the data.
 #'
-#' @param cube Hypercube for which the selection criteria will be defined.
+#' @param x Hypercube for which the selection criteria will be defined.
 #' @param dimensions A vector of dimensions that are used in the aggregation.
 #' @param fun The function that is used for aggregation. Possible functions are sum, prod, min, max, mean, median, sd, and count.
 #' @return Returns a \code{Cube} object with the added aggregation.
 #' @author Michael Scholz \email{michael.scholz@@uni-passau.de}
 #' @seealso \code{\link[=Cube-class]{Cube}} \code{\link{remove.aggregation}} \code{\link{add.selection}}
+#' @aliases add.aggregation,Cube-method
+#' @keywords methods
+#' @docType methods
 #' @examples
 #'
 #' data("sales")
@@ -342,33 +449,35 @@ remove.selection = function(cube, dimensions) {
 #' cube
 #'
 #' @export add.aggregation
-add.aggregation = function(cube, dimensions, fun = c("sum", "min", "max", "prod", "mean", "median", "sd", "count")) {
-  if (!is.object(cube)) {
-    stop("Parameter cube must be of type Cube.")
+setGeneric("add.aggregation", function(x, dimensions, fun = c("sum", "min", "max", "prod", "mean", "median", "sd", "count"))
+  standardGeneric("add.aggregation"))
+setMethod("add.aggregation", "Cube", function(x, dimensions, fun = c("sum", "min", "max", "prod", "mean", "median", "sd", "count")) {
+  if (!is.object(x)) {
+    stop("Parameter x must be of type Cube.")
   }
-  if (class(cube)[[1]] != "Cube") {
-    stop("Parameter cube must be of type Cube.")
+  if (class(x)[[1]] != "Cube") {
+    stop("Parameter x must be of type Cube.")
   }
   if (!is.vector(dimensions)) {
     stop("Parameter dimensions must be a vector.")
   }
   fun = match.arg(fun)
-  dims = unlist(cube@structure)
+  dims = unlist(x@structure)
   for (dimension in dimensions) {
     if (dimension %in% dims) {
-      cube@view[[dimension]]@aggregation = c(cube@view[[dimension]]@aggregation, fun)
-      cube@view[[dimension]]@aggregation = cube@view[[dimension]]@aggregation[which(cube@view[[dimension]]@aggregation != "")]
+      x@view[[dimension]]@aggregation = c(x@view[[dimension]]@aggregation, fun)
+      x@view[[dimension]]@aggregation = x@view[[dimension]]@aggregation[which(x@view[[dimension]]@aggregation != "")]
     } else {
       stop(paste("There is no dimension", dimension, "in your cube."))
     }
   }
   nonAggregation = dims[which(!dims %in% dimensions)]
   for (dimension in nonAggregation) {
-    cube@view[[dimension]]@aggregation = c(cube@view[[dimension]]@aggregation, "none")
-    cube@view[[dimension]]@aggregation = cube@view[[dimension]]@aggregation[which(cube@view[[dimension]]@aggregation != "")]
+    x@view[[dimension]]@aggregation = c(x@view[[dimension]]@aggregation, "none")
+    x@view[[dimension]]@aggregation = x@view[[dimension]]@aggregation[which(x@view[[dimension]]@aggregation != "")]
   }
-  return(cube)
-}
+  return(x)
+})
 
 #' Removes aggregations from a hypercube
 #'
@@ -376,12 +485,15 @@ add.aggregation = function(cube, dimensions, fun = c("sum", "min", "max", "prod"
 #' The cube itself will not be changed. The aggregation only affect the data that
 #' will be shown when printing the cube.
 #'
-#' @param cube Hypercube from which the aggregation will be removed.
+#' @param x Hypercube from which the aggregation will be removed.
 #' @param dimensions A vector of dimensions for which the aggregations will be removed.
 #' @param last Should the last aggregation be removed? If this parameter is set TRUE, the dimension vector will be ignored.
 #' @return Returns a \code{Cube} object with the added aggregation.
 #' @author Michael Scholz \email{michael.scholz@@uni-passau.de}
 #' @seealso \code{\link[=Cube-class]{Cube}} \code{\link{add.aggregation}} \code{\link{remove.selection}}
+#' @aliases remove.aggregation,Cube-method
+#' @keywords methods
+#' @docType methods
 #' @examples
 #'
 #' data("sales")
@@ -395,31 +507,33 @@ add.aggregation = function(cube, dimensions, fun = c("sum", "min", "max", "prod"
 #' cube
 #'
 #' @export remove.aggregation
-remove.aggregation = function(cube, dimensions = NA, last = FALSE) {
-  if (!is.object(cube)) {
-    stop("Parameter cube must be of type Cube.")
+setGeneric("remove.aggregation", function(x, dimensions = NA, last = FALSE)
+  standardGeneric("remove.aggregation"))
+setMethod("remove.aggregation", "Cube", function(x, dimensions = NA, last = FALSE) {
+  if (!is.object(x)) {
+    stop("Parameter x must be of type Cube.")
   }
-  if (class(cube)[[1]] != "Cube") {
-    stop("Parameter cube must be of type Cube.")
+  if (class(x)[[1]] != "Cube") {
+    stop("Parameter x must be of type Cube.")
   }
   if (!is.vector(dimensions)) {
     if (!last)
       stop("Parameter dimensions must be a vector.")
   }
-  dims = unlist(cube@structure)
-  maxLength = max(unlist(lapply(cube@view, FUN = function(x) return(length(x@aggregation)))))
+  dims = unlist(x@structure)
+  maxLength = max(unlist(lapply(x@view, FUN = function(x) return(length(x@aggregation)))))
   if (last) {
     for (j in dims) {
-      cube@view[[j]]@aggregation = cube@view[[j]]@aggregation[1:(maxLength-1)]
+      x@view[[j]]@aggregation = x@view[[j]]@aggregation[1:(maxLength-1)]
     }
     maxLength = maxLength - 1
   } else {
     for (dimension in dimensions) {
       if (dimension %in% dims) {
         if (maxLength < 2) {
-          cube@view[[dimension]]@aggregation = "none"
+          x@view[[dimension]]@aggregation = "none"
         } else {
-          cube@view[[dimension]]@aggregation = rep("none", maxLength)
+          x@view[[dimension]]@aggregation = rep("none", maxLength)
         }
       } else {
         stop(paste("There is no dimension", dimension, "in your cube."))
@@ -427,60 +541,65 @@ remove.aggregation = function(cube, dimensions = NA, last = FALSE) {
     }
   }
   for (i in maxLength:1) {
-    s = sum(unlist(lapply(cube@view, FUN = function(x) return(x@aggregation[i] == "none" || x@aggregation[i] == ""))))
+    s = sum(unlist(lapply(x@view, FUN = function(x) return(x@aggregation[i] == "none" || x@aggregation[i] == ""))))
     if (s == length(dims)) {
       for (j in dims) {
           if (i == 1) {
-            cube@view[[j]]@aggregation = ""
+            x@view[[j]]@aggregation = ""
           } else {
-            cube@view[[j]]@aggregation = cube@view[[j]]@aggregation[1:(i-1)]
+            x@view[[j]]@aggregation = x@view[[j]]@aggregation[1:(i-1)]
           }
       }
     }
   }
-  return(cube)
-}
+  return(x)
+})
 
 
 #' Changes the order of the dimensions in a given cube
 #'
-#' @param cube Hypercube for which the dimension should be re-ordered.
+#' @param x Hypercube for which the dimensions should be re-ordered.
 #' @param dimensions Vector of dimensions. The order of the dimensions in this vector defines the order of the dimensions in the cube.
 #' @return Returns a \code{Cube} object.
 #' @author Michael Scholz \email{michael.scholz@@uni-passau.de}
 #' @seealso \code{\link[=Cube-class]{Cube}}
+#' @aliases change.dimensionOrder,Cube-method
+#' @keywords methods
+#' @docType methods
 #' @examples
 #'
 #' data("sales")
 #' cube = generateCube(sales, columns = list(time = c("month", "year"),
 #'       location = c("state"), product = "product"), valueColumn = "amount")
-#' cube = changeDimensionOrder(cube, dimensions = c("product", "month", "year", "state"))
+#' cube = change.dimensionOrder(cube, dimensions = c("product", "month", "year", "state"))
 #' cube
 #'
-#' @export changeDimensionOrder
-changeDimensionOrder = function(cube, dimensions) {
-  if (!is.object(cube)) {
-    stop("Parameter cube must be of type Cube.")
+#' @export change.dimensionOrder
+setGeneric("change.dimensionOrder", function(x, dimensions)
+  standardGeneric("change.dimensionOrder"))
+setMethod("change.dimensionOrder", "Cube", function(x, dimensions) {
+  if (!is.object(x)) {
+    stop("Parameter x must be of type Cube.")
   }
-  if (class(cube)[[1]] != "Cube") {
-    stop("Parameter cube must be of type Cube.")
+  if (class(x)[[1]] != "Cube") {
+    stop("Parameter x must be of type Cube.")
   }
   if (!is.vector(dimensions)) {
     stop("Parameter dimensions must be a vector.")
   }
-  if (names(cube@view) %in% dimensions && length(names(cube@view)) == length(dimensions)) {
-    cube@view = cube@view[dimensions]
+  if (all(names(x@view) %in% dimensions) && length(names(x@view)) == length(dimensions)) {
+    x@view = x@view[dimensions]
   } else {
     stop("Parameter dimensions must be a vector containing the names of all dimensions of your cube.")
   }
-  return(cube)
-}
+  return(x)
+})
 
 
 #' Converts the actual view of a cube to a data frame
 #'
 #' Converts the actual view of a \code{Cube} object to a data frame. All added selections and
-#' aggregations will be regarded. Note that selection criteria will be applied before the
+#' aggregations will be regarded. Note that selection criteria will be applied before
 #' aggregating the data.
 #'
 #' @param x The \code{Cube} object that will be converted to a data frame.
@@ -495,7 +614,7 @@ changeDimensionOrder = function(cube, dimensions) {
 #' data("sales")
 #' cube = generateCube(sales, columns = list(time = c("month", "year"),
 #'       location = c("state"), product = "product"), valueColumn = "amount")
-#' cube = changeDimensionOrder(cube, dimensions = c("product", "month", "year", "state"))
+#' cube = change.dimensionOrder(cube, dimensions = c("product", "month", "year", "state"))
 #' df = as.data.frame(cube)
 #' df
 #'
@@ -528,3 +647,102 @@ as.data.frame.Cube = function(x, row.names = NULL, optional = FALSE, ...) {
   names(df)[ncol(df)] = "value"
   return(df)
 }
+
+
+#' Calculates the sparsity of a given cube.
+#'
+#' Calculates the sparsity of the actual view of a \code{Cube} object. All added selections and
+#' aggregations will be regarded. Note that selection criteria will be applied before
+#' aggregating the data.
+#'
+#' @param x The \code{Cube} object for which the sparsity will be computed.
+#' @author Michael Scholz \email{michael.scholz@@uni-passau.de}
+#' @return Sparsity value
+#' @seealso \code{\link{importance}}
+#' @aliases sparsity,Cube-method
+#' @keywords methods
+#' @docType methods
+#' @examples
+#'
+#' data("sales")
+#' cube = generateCube(sales, columns = list(time = c("month", "year"),
+#'       location = c("state"), product = "product"), valueColumn = "amount")
+#' sparsity(cube)
+#'
+#' @export
+setGeneric("sparsity", function(x)
+  standardGeneric("sparsity"))
+setMethod("sparsity", "Cube", function(x) {
+  df = as.data.frame(x)
+  sparsity = sum(is.na(df[,ncol(df)])) / nrow(df)
+  return(sparsity)
+})
+
+
+#' Calculates the dimension importances of a given cube.
+#'
+#' Calculates the importance values for all dimensions of the actual view of a \code{Cube} object. All added selections and
+#' aggregations will be regarded. Note that selection criteria will be applied before
+#' aggregating the data.
+#'
+#' @param x The \code{Cube} object for which the importance values will be computed.
+#' @author Michael Scholz \email{michael.scholz@@uni-passau.de}
+#' @return Sparsity value
+#' @seealso \code{\link{sparsity}}
+#' @aliases importance,Cube-method
+#' @keywords methods
+#' @docType methods
+#' @examples
+#'
+#' data("sales")
+#' cube = generateCube(sales, columns = list(time = c("month", "year"),
+#'       location = c("state"), product = "product"), valueColumn = "amount")
+#' importance(cube)
+#'
+#' @export
+setGeneric("importance", function(x)
+  standardGeneric("importance"))
+setMethod("importance", "Cube", function(x) {
+  data = as.data.frame(x)
+  target = "value"
+  data = data[!is.na(data$value),]
+  features = names(data)
+  features = as.matrix(features[-which(features == target)], ncol = 1)
+  imp = apply(features, 1, .informationGain, data, target)
+  imp = as.matrix(imp/sum(imp), ncol = 1)
+  rownames(imp) = features
+  importances = lapply(x@structure, .dimensionImportance, imp)
+  class(importances) = "Importances"
+  return(importances)
+})
+
+
+#' Prints an Importances object.
+#'
+#' Prints an \code{Importances} object.
+#'
+#' @param x The \code{Importances} object that will be printed.
+#' @param ... Ignored parameters.
+#' @author Michael Scholz \email{michael.scholz@@uni-passau.de}
+#' @return Sparsity value
+#' @seealso \code{\link{importance}}
+#' @examples
+#'
+#' data("sales")
+#' cube = generateCube(sales, columns = list(time = c("month", "year"),
+#'       location = c("state"), product = "product"), valueColumn = "amount")
+#' importances = importance(cube)
+#' print(importances)
+#'
+#' @export
+print.Importances = function(x, ...) {
+  for (i in 1:length(x)) {
+    cat(names(x)[i], ": ", x[[i]]$importance, "\n", sep = "")
+    attributes = x[[i]]$attributes
+    if (length(attributes) > 1) {
+      for (a in 1:length(attributes)) {
+        cat("- ", rownames(attributes)[a], ": ", attributes[a], "\n", sep = "")
+      }
+    }
+  }
+}
